﻿using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Windows.Input;
using Zeta.Common.Xml;
using Zeta.XmlEngine;

namespace Armory.Settings
{
    [XmlElement("AmorySettings")]
    public class ArmorySettings : XmlSettings
    {
        #region Singleton
        private static ArmorySettings _instance;

        public ArmorySettings() :
            base(Path.Combine(Path.Combine(SettingsDirectory, "Armory"), "ArmorySettings.xml"))
        {

        }

        public static ArmorySettings Instance
        {
            get { return _instance ?? (_instance = new ArmorySettings()); }
        }
        #endregion

        #region fields
        private bool disableAt70;
        private bool identifyItems;
        private bool checkStash;

        private bool _UseBasicUpgradeFormula;
        private float basicUpgradeFormulaDamage;
        private float basicUpgradeFormulaToughness;

        private bool useAdditiveDamageFormula;
        private float additiveDamageFormulaDamage;
        private float additiveDamageFormulaDamageToughness;

        private bool useHealingUpgradeFormula;
        private float healingUpgradeFormulaDamage;
        private float healingUpgradeFormulaDamageToughness;
        private float healingUpgradeFormulaHealing;

        private ProtectedSlots protectedSlots;

        private ICommand resetCommand;
        #endregion

        /// <summary>
        /// Default: false
        /// </summary>
        [XmlElement("DisableAt70")]
        [Setting]
        [DefaultValue(false)]
        public bool DisableAt70
        {
            get { return disableAt70; }
            set
            {
                if (disableAt70 != value)
                {
                    disableAt70 = value;
                    OnPropertyChanged("DisableAt70");
                }
            }
        }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("IdentifyItems")]
        [Setting]
        [DefaultValue(true)]
        public bool IdentifyItems
        {
            get { return identifyItems; }
            set
            {
                if (identifyItems != value)
                {
                    identifyItems = value;
                    OnPropertyChanged("IdentifyItems");
                }
            }
        }


        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("CheckStash")]
        [Setting]
        [DefaultValue(true)]
        public bool CheckStash
        {
            get { return checkStash; }
            set
            {
                if (checkStash != value)
                {
                    checkStash = value;
                    OnPropertyChanged("CheckStash");
                }
            }
        }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("UseBasicUpgradeFormula")]
        [Setting]
        [DefaultValue(true)]
        public bool UseBasicUpgradeFormula
        {
            get { return _UseBasicUpgradeFormula; }
            set
            {
                if (_UseBasicUpgradeFormula != value)
                {
                    _UseBasicUpgradeFormula = value;
                    OnPropertyChanged("UseBasicUpgradeFormula");
                }
            }
        }

        /// <summary>
        /// Default: 0
        /// </summary>
        [XmlElement("BasicUpgradeFormulaDamage")]
        [Setting]
        [DefaultValue(0)]
        public float BasicUpgradeFormulaDamage
        {
            get { return basicUpgradeFormulaDamage; }
            set
            {
                if (basicUpgradeFormulaDamage != value)
                {
                    basicUpgradeFormulaDamage = value;
                    OnPropertyChanged("BasicUpgradeFormulaDamage");
                }
            }
        }

        /// <summary>
        /// Default: 0
        /// </summary>
        [XmlElement("BasicUpgradeFormulaToughness")]
        [Setting]
        [DefaultValue(0)]
        public float BasicUpgradeFormulaToughness
        {
            get { return basicUpgradeFormulaToughness; }
            set
            {
                if (basicUpgradeFormulaToughness != value)
                {
                    basicUpgradeFormulaToughness = value;
                    OnPropertyChanged("BasicUpgradeFormulaToughness");
                }
            }
        }


        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("UseAdditiveDamageFormula")]
        [Setting]
        [DefaultValue(true)]
        public bool UseAdditiveDamageFormula
        {
            get { return useAdditiveDamageFormula; }
            set
            {
                if (useAdditiveDamageFormula != value)
                {
                    useAdditiveDamageFormula = value;
                    OnPropertyChanged("UseAdditiveDamageFormula");
                }
            }
        }

        /// <summary>
        /// Default: -0.05f
        /// </summary>
        [XmlElement("AdditiveDamageFormulaDamage")]
        [Setting]
        [DefaultValue(-0.05f)]
        public float AdditiveDamageFormulaDamage
        {
            get { return additiveDamageFormulaDamage; }
            set
            {
                if (additiveDamageFormulaDamage != value)
                {
                    additiveDamageFormulaDamage = value;
                    OnPropertyChanged("AdditiveDamageFormulaDamage");
                }
            }
        }

        /// <summary>
        /// Default: 0
        /// </summary>
        [XmlElement("AdditiveDamageFormulaDamageToughness")]
        [Setting]
        [DefaultValue(0)]
        public float AdditiveDamageFormulaDamageToughness
        {
            get { return additiveDamageFormulaDamageToughness; }
            set
            {
                if (additiveDamageFormulaDamageToughness != value)
                {
                    additiveDamageFormulaDamageToughness = value;
                    OnPropertyChanged("AdditiveDamageFormulaDamageToughness");
                }
            }
        }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("UseHealingUpgradeFormula")]
        [Setting]
        [DefaultValue(true)]
        public bool UseHealingUpgradeFormula
        {
            get { return useHealingUpgradeFormula; }
            set
            {
                if (useHealingUpgradeFormula != value)
                {
                    useHealingUpgradeFormula = value;
                    OnPropertyChanged("UseHealingUpgradeFormula");
                }
            }
        }


        /// <summary>
        /// Default: -0.15f
        /// </summary>
        [XmlElement("HealingUpgradeFormulaDamage")]
        [Setting]
        [DefaultValue(-0.15f)]
        public float HealingUpgradeFormulaDamage
        {
            get { return healingUpgradeFormulaDamage; }
            set
            {
                if (healingUpgradeFormulaDamage != value)
                {
                    healingUpgradeFormulaDamage = value;
                    OnPropertyChanged("HealingUpgradeFormulaDamage");
                }
            }
        }

        /// <summary>
        /// Default: 0
        /// </summary>
        [XmlElement("HealingUpgradeFormulaDamageToughness")]
        [Setting]
        [DefaultValue(0)]
        public float HealingUpgradeFormulaDamageToughness
        {
            get { return healingUpgradeFormulaDamageToughness; }
            set
            {
                if (healingUpgradeFormulaDamageToughness != value)
                {
                    healingUpgradeFormulaDamageToughness = value;
                    OnPropertyChanged("HealingUpgradeFormulaDamageToughness");
                }
            }
        }

        /// <summary>
        /// Default: 0
        /// </summary>
        [XmlElement("HealingUpgradeFormulaHealing")]
        [Setting]
        [DefaultValue(0)]
        public float HealingUpgradeFormulaHealing
        {
            get { return healingUpgradeFormulaHealing; }
            set
            {
                if (healingUpgradeFormulaHealing != value)
                {
                    healingUpgradeFormulaHealing = value;
                    OnPropertyChanged("HealingUpgradeFormulaHealing");
                }
            }
        }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("AllowTwoHand")]
        [Setting]
        [DefaultValue(true)]
        public bool AllowTwoHand { get; set; }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("AllowShields")]
        [Setting]
        [DefaultValue(true)]
        public bool AllowShields { get; set; }

        /// <summary>
        /// Default: true
        /// </summary>
        [XmlElement("DualWield")]
        [Setting]
        [DefaultValue(true)]
        public bool DualWield { get; set; }

        public ICommand ResetCommand
        {
            get
            {
                if (resetCommand == null)
                {
                    resetCommand = new RelayCommand((parameter) =>
                    {
                        ArmorySettings.Instance.SetDefaults();
                    });
                }

                return resetCommand;
            }
            private set
            {
                resetCommand = value;
            }
        }

        public ProtectedSlots ProtectedSlots
        {
            get
            {
                return ProtectedSlots.Instance;
            }
            set
            {
                ProtectedSlots.Instance = value;
            }
        }
    }
}
